/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/dataset/utils/DataSetTypeFilter.h

   \brief Custom widget used filter a dataset type, creating a DataSetTypeConverter.
 */

#include "DataSetTypeFilter.h"

#include "../../utils/DoubleListWidget.h"
#include "../../../../dataaccess/utils/Utils.h"
#include "../../../../geometry/GeometryProperty.h"

te::qt::widgets::DataSetTypeFilter::DataSetTypeFilter(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f)
{
  //add double list widget to this form
  m_doubleListWidget.reset(new te::qt::widgets::DoubleListWidget(this));
  m_doubleListWidget->setLeftLabel(tr("Input Attributes"));
  m_doubleListWidget->setRightLabel(tr("Output Attributes"));
  m_doubleListWidget->setVisibledOrderButtons(false);

  QGridLayout* layout = new QGridLayout(this);
  layout->addWidget(m_doubleListWidget.get());
  layout->setContentsMargins(0, 0, 0, 0);
}

te::qt::widgets::DataSetTypeFilter::~DataSetTypeFilter() = default;

void te::qt::widgets::DataSetTypeFilter::setDataSetType(const te::da::DataSetType* dsType)
{
  m_dsType.reset(dynamic_cast<te::da::DataSetType*>(dsType->clone()));

  m_doubleListWidget->clearInputValues();
  m_doubleListWidget->clearOutputValues();

  std::set<std::string> fixedProperties;

  //added as fixed output pk properties
  if (m_dsType->getPrimaryKey())
  {
    const std::vector<te::dt::Property*>& properties = m_dsType->getPrimaryKey()->getProperties();

    std::vector<std::string> pkProperties;

    for (size_t t = 0; t < properties.size(); ++t)
    {
      pkProperties.push_back(properties[t]->getName());

      fixedProperties.insert(properties[t]->getName());
    }

    m_doubleListWidget->setFixedOutputValues(pkProperties, "key");
  }

  //added as fixed output geom prop
  te::gm::GeometryProperty* geomProp = te::da::GetFirstGeomProperty(m_dsType.get());

  if (geomProp)
  {
    std::vector<std::string> geomProperties;
    geomProperties.push_back(geomProp->getName());

    fixedProperties.insert(geomProp->getName());

    m_doubleListWidget->setFixedOutputValues(geomProperties, "geometry");
  }

  //add other properties as input
  std::vector<te::dt::Property*> props = m_dsType->getProperties();

  std::vector<std::string> inputValues;

  for (std::size_t i = 0; i < props.size(); ++i)
  {
    std::set<std::string>::iterator it = fixedProperties.find(props[i]->getName());

    if (it == fixedProperties.end())
    {
      inputValues.push_back(props[i]->getName());
    }
  }

  m_doubleListWidget->setInputValues(inputValues);
}

std::unique_ptr<te::da::DataSetTypeConverter> te::qt::widgets::DataSetTypeFilter::getConverter(std::string name)
{
  std::unique_ptr<te::da::DataSetTypeConverter> converter(new te::da::DataSetTypeConverter(m_dsType.get()));

  te::da::DataSetType* dsTypeResult = converter->getResult();

  dsTypeResult->setTitle(name);
  
  std::vector<std::string> attrs = m_doubleListWidget->getOutputValues();

  bool hasGeomProp = false;

  te::gm::GeometryProperty* geomProp = nullptr;

  for (std::size_t t = 0; t < attrs.size(); ++t)
  {
    te::dt::Property* prop = m_dsType->getProperty(attrs[t]);

    if (prop->getType() == te::dt::GEOMETRY_TYPE)
    {
      geomProp = dynamic_cast<te::gm::GeometryProperty*>(prop);
      continue;
    }

    te::dt::Property* propClone = prop->clone();

    std::vector<std::string> propVec;
    propVec.push_back(attrs[t]);

    converter->add(propVec, propClone);
  }

  if (geomProp != nullptr)
  {
    te::dt::Property* propClone = geomProp->clone();

    std::vector<std::string> propVec;
    propVec.push_back(geomProp->getName());

    converter->add(propVec, propClone);
  }

  //create primary key
  if (m_dsType->getPrimaryKey())
  {
    te::da::PrimaryKey* pk = new te::da::PrimaryKey(dsTypeResult);

    std::string pkName = name + "_" + m_dsType->getPrimaryKey()->getName() + "_pk";

    pk->setName(pkName);

    const std::vector<te::dt::Property*>& properties = m_dsType->getPrimaryKey()->getProperties();

    for (size_t t = 0; t < properties.size(); ++t)
    {
      pk->add(properties[t]);
    }
  }

  //create index
  geomProp = te::da::GetFirstGeomProperty(dsTypeResult);

  if (geomProp)
  {
    te::da::Index* idx = new te::da::Index(dsTypeResult);

    std::string nameIndex = name + "_" + geomProp->getName() + "_idx";

    idx->setName(nameIndex);
    idx->setIndexType(te::da::R_TREE_TYPE);

    idx->add(geomProp);
  }

  return converter;
}

