/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/se/LineSymbolizerProperty.cpp

  \brief A widget used to define the line symbolizer
         properties, such as:
           - basic stroke
*/

// TerraLib
#include "../../../se/LineSymbolizer.h"
#include "../../../se/Stroke.h"
#include "../../../se/GraphicStroke.h"
#include "../propertybrowser/AbstractPropertyManager.h"
#include "BasicStrokePropertyItem.h"
#include "LineSymbolizerProperty.h"
#include "GraphicProperty.h"
#include "../../../se/ParameterValue.h"
#include "LinePerpendicularOffsetPropertyItem.h"
#include "LineInitialGapPropertyItem.h"
#include "LineGapPropertyItem.h"
#include "../../../se/Utils.h"

// Qt
#include <QGridLayout>
#include <QToolBox>

// QtPropertyBrowser
#include <QtPropertyBrowser/QtTreePropertyBrowser>
#include <QtPropertyBrowser/QtVariantPropertyManager>

// STL
#include <cassert>

te::qt::widgets::LineSymbolizerProperty::LineSymbolizerProperty(
    QWidget* parent): 
  AbstractSymbolizerProperty(parent),
  m_setLocalSymbol(false), m_symb(nullptr)
{
  QGridLayout* layout = new QGridLayout(this);
  this->setLayout(layout);

  QToolBox* tb = new QToolBox(this);
  layout->addWidget(tb);

  QtTreePropertyBrowser* basicPropBrowser = new QtTreePropertyBrowser(this);
  basicPropBrowser->setIndentation(10);
  basicPropBrowser->setPropertiesWithoutValueMarked(true);
  basicPropBrowser->setRootIsDecorated(false);
  basicPropBrowser->setResizeMode(QtTreePropertyBrowser::ResizeToContents);

  layout->addWidget(basicPropBrowser);

  // Graphic Stroke configuration

  m_bs = new te::qt::widgets::BasicStrokePropertyItem(basicPropBrowser);

  // perpendicular offset: a widget used to define the Perpendicular Offset between a line and marks.
  m_perpendicularOffset = new te::qt::widgets::LinePerpendicularOffsetPropertyItem(basicPropBrowser);

  // initial gap: a widget used to define the Initial Gap to draw marks in a line.
  m_initialGap = new te::qt::widgets::LineInitialGapPropertyItem(basicPropBrowser);

  // gap: a widget used to define the gap between marks.
  m_gap = new te::qt::widgets::LineGapPropertyItem(basicPropBrowser);

  tb->addItem(basicPropBrowser, tr("Basic Symbology"));

  // A widget that will keep the Graphic Stroke mark configuration
  m_graph = new te::qt::widgets::GraphicProperty(this);

  tb->addItem(m_graph, tr("Graphic Stroke"));

  connect(m_bs, SIGNAL(strokeChanged()), this, SLOT(onStrokeChanged()));
  connect(m_graph, SIGNAL(graphicChanged()), this, SLOT(onLineGraphicChanged()));
  connect(m_perpendicularOffset, SIGNAL(linePerpendicularOffsetChanged()), this, SLOT(onLinePerpendicularOffsetChanged()));
  connect(m_initialGap, SIGNAL(lineInitialGapChanged()), this, SLOT(onInitialGapChanged()));
  connect(m_gap, SIGNAL(lineGapChanged()), this, SLOT(onGapChanged()));
}

te::qt::widgets::LineSymbolizerProperty::~LineSymbolizerProperty()
{
  delete m_bs;
  delete m_perpendicularOffset;
  delete m_initialGap;
  delete m_gap;
  delete m_graph;
}

void te::qt::widgets::LineSymbolizerProperty::setSymbolizer(te::se::Symbolizer* symb)
{
  m_symb = dynamic_cast<te::se::LineSymbolizer*>(symb);

  m_setLocalSymbol = true;

  if (m_symb->getStroke())
  {
    m_bs->setStroke(m_symb->getStroke());
    
    if (m_symb->getStroke()->getGraphicStroke() != nullptr)
    {
      te::se::GraphicStroke* graphicStroke = m_symb->getStroke()->getGraphicStroke()->clone();

      setGraphicStroke(graphicStroke);
    }
  }    

  if (m_symb->getPerpendicularOffset())
  {
    // perpendicular offset: a widget used to define the Perpendicular Offset between a line and marks.
    m_perpendicularOffset->setLinePerpendicularOffset(te::se::GetDouble(m_symb->getPerpendicularOffset()));
  }
  
  m_setLocalSymbol = false;
}

te::se::Symbolizer* te::qt::widgets::LineSymbolizerProperty::getSymbolizer() const
{
  return m_symb;
}

void te::qt::widgets::LineSymbolizerProperty::setGraphicStroke(te::se::GraphicStroke* graphicStroke)
{
  if (graphicStroke == nullptr)
  {
    return;
  }

  // initial gap: a widget used to define the Initial Gap to draw marks in a line.
  if (graphicStroke->getInitialGap())
  {
    m_initialGap->setLineInitialGap(te::se::GetDouble(graphicStroke->getInitialGap()));
  }

  // gap: a widget used to define the gap between marks.
  if(graphicStroke->getGap())
  {
    m_gap->setLineGap(te::se::GetDouble(graphicStroke->getGap()));
  }

  // A widget that will keep the Graphic Stroke mark configuration
  if (graphicStroke->getGraphic())
  {
    m_graph->setGraphic(graphicStroke->getGraphic());
  }
}

void te::qt::widgets::LineSymbolizerProperty::onStrokeChanged()
{
  if (m_setLocalSymbol)
    return;

  m_symb->setStroke(m_bs->getStroke());

  emit symbolizerChanged();
}

void te::qt::widgets::LineSymbolizerProperty::onLineGraphicChanged()
{
  if (m_setLocalSymbol)
    return;

  te::se::GraphicStroke* newGraphicStroke = nullptr;

  te::se::Stroke* stroke = m_bs->getStroke();

  const te::se::GraphicStroke* graphicStroke = stroke->getGraphicStroke();
  if (graphicStroke == nullptr)
  {
    newGraphicStroke = new te::se::GraphicStroke();
  }
  else
  {
    newGraphicStroke = stroke->getGraphicStroke()->clone();
  }

  newGraphicStroke->setGraphic(m_graph->getGraphic());
  stroke->setGraphicStroke(newGraphicStroke);

  m_bs->setStroke(stroke);

  m_symb->setStroke(m_bs->getStroke());

  emit symbolizerChanged();
}

void te::qt::widgets::LineSymbolizerProperty::onLinePerpendicularOffsetChanged()
{
  if (m_setLocalSymbol)
    return;

  double offset = m_perpendicularOffset->getLinePerpendicularOffset();

  m_symb->setPerpendicularOffset(new te::se::ParameterValue(QString::number(offset, 'g', 2).toUtf8().data()));

  emit symbolizerChanged();
}

void te::qt::widgets::LineSymbolizerProperty::onInitialGapChanged()
{
  if (m_setLocalSymbol)
    return;

  te::se::GraphicStroke* newGraphicStroke = nullptr;

  te::se::Stroke* stroke = m_bs->getStroke();

  const te::se::GraphicStroke* graphicStroke = stroke->getGraphicStroke();
  if (graphicStroke == nullptr)
  {
    newGraphicStroke = new te::se::GraphicStroke();
  }
  else
  {
    newGraphicStroke = stroke->getGraphicStroke()->clone();
  }

  double initialGap = m_initialGap->getLineInitialGap();

  newGraphicStroke->setInitialGap(new te::se::ParameterValue(QString::number(initialGap, 'g', 2).toUtf8().data()));
  stroke->setGraphicStroke(newGraphicStroke);

  m_bs->setStroke(stroke);

  m_symb->setStroke(m_bs->getStroke());

  emit symbolizerChanged();
}

void te::qt::widgets::LineSymbolizerProperty::onGapChanged()
{
  if (m_setLocalSymbol)
    return;

  te::se::GraphicStroke* newGraphicStroke = nullptr;

  te::se::Stroke* stroke = m_bs->getStroke();

  const te::se::GraphicStroke* graphicStroke = stroke->getGraphicStroke();
  if (graphicStroke == nullptr)
  {
    newGraphicStroke = new te::se::GraphicStroke();
  }
  else
  {
    newGraphicStroke = stroke->getGraphicStroke()->clone();
  }

  double gap = m_gap->getLineGap();

  newGraphicStroke->setGap(new te::se::ParameterValue(QString::number(gap, 'g', 2).toUtf8().data()));
  stroke->setGraphicStroke(newGraphicStroke);

  m_bs->setStroke(stroke);

  m_symb->setStroke(m_bs->getStroke());

  emit symbolizerChanged();
}
