/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/se/LocalImagePropertyItem.h

  \brief A widget used to define a local image graphic for a se symbolizer.
*/

// TerraLib
#include "../../../se/Config.h"
#include "../../../se/ExternalGraphic.h"
#include "../../../se/InlineContent.h"
#include "../../../xlink/SimpleLink.h"
#include "../propertybrowser/AbstractPropertyManager.h"
#include "LocalImagePropertyItem.h"
#include "ExternalGraphicRenderer.h"

// STL
#include <cassert>

// Qt
#include <QBuffer>
#include <QDialog>
#include <QFileDialog>
#include <QGridLayout>
#include <QImageReader>
#include <QMessageBox>
#include <QPainter>
#include <QPushButton>
#include <QScrollArea>
#include <QSvgRenderer>

// QtPropertyBrowser
#include <QtPropertyBrowser/QtTreePropertyBrowser>
#include <QtPropertyBrowser/QtVariantPropertyManager>


te::qt::widgets::LocalImagePropertyItem::LocalImagePropertyItem(QtTreePropertyBrowser* pb, QColor c) : te::qt::widgets::AbstractPropertyItem(pb, c) ,
  m_eg(new te::se::ExternalGraphic), m_update(false)
{
  //build property browser basic fill
  QtProperty* localImgProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_groupManager->addProperty(tr("External Graphic"));

  //onLineResource
  m_onLineResourceProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->addProperty(tr("OnLine Resource"));
  te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->setValue(m_onLineResourceProperty, QString(""));
  localImgProperty->addSubProperty(m_onLineResourceProperty);

  //inLineContent
  m_inLineContentProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->addProperty(tr("InLine Content"));
  te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->setValue(m_inLineContentProperty, tr("NOT DEFINED"));
  localImgProperty->addSubProperty(m_inLineContentProperty);

  connect(te::qt::widgets::AbstractPropertyManager::getInstance().m_dlgFactory, SIGNAL(internalDlg(QWidget *, QtProperty *)), this, SLOT(setDlg(QWidget *, QtProperty *)));

  addProperty(localImgProperty, tr("External Graphic"), QColor(255, 190, 160));
}

te::qt::widgets::LocalImagePropertyItem::~LocalImagePropertyItem()
{
  delete m_eg;

  delete m_onLineResourceProperty;
  delete m_inLineContentProperty;
}

void te::qt::widgets::LocalImagePropertyItem::setExternalGraphic(const te::se::ExternalGraphic* eg)
{
  assert(eg);

  delete m_eg;

  m_eg = eg->clone();

  updateUi();
}

te::se::ExternalGraphic* te::qt::widgets::LocalImagePropertyItem::getExternalGraphic() const
{
  return m_eg->clone();
}

void te::qt::widgets::LocalImagePropertyItem::updateUi()
{
  // Updating path
  const te::xl::SimpleLink* link = m_eg->getOnlineResource();

  if (link)
  {
    QString path = link->getHref().c_str();

    te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->setValue(m_onLineResourceProperty, path);
    te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->setValue(m_inLineContentProperty, tr("NOT DEFINED"));
  }

  const te::se::InlineContent* inLineContent = m_eg->getInlineContent();

  if (inLineContent)
  {
    te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->setValue(m_inLineContentProperty, tr("DEFINED"));
    te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->setValue(m_onLineResourceProperty, "");
  }
}

void te::qt::widgets::LocalImagePropertyItem::valueChanged(
    QtProperty* /*p*/, const QString& /*value*/)
{

}

void te::qt::widgets::LocalImagePropertyItem::setDlg(QWidget *parent, QtProperty * prop)
{
  if(prop == m_onLineResourceProperty)
  {
    m_dlg = parent;

    connect(m_dlg, SIGNAL(showDlg()), this, SLOT(showOnLineResourceDlg()));
  }
  else if (prop == m_inLineContentProperty)
  {
    m_dlg = parent;

    connect(m_dlg, SIGNAL(showDlg()), this, SLOT(showInLineContentDlg()));
  }
}

void te::qt::widgets::LocalImagePropertyItem::showOnLineResourceDlg()
{
  // Bulding the filter string
  QString filter = tr("Images") + " ( ";
  QList<QByteArray> formats = QImageReader::supportedImageFormats();
  for(int i = 0; i < formats.size() - 1; ++i)
    filter += "*." + formats[i] + " ";
  filter += ")";

  QString path = QFileDialog::getOpenFileName(this, tr("Select an Image File"), "", filter);
  if(path.isNull())
    return;

  // Updating external graphic path
  te::xl::SimpleLink* link = new te::xl::SimpleLink;
  link->setHref(path.toUtf8().data());
  m_eg->setOnlineResource(link);
  m_eg->setInlineContent(nullptr);

  // Updating external graphic format
  QString f(QImageReader::imageFormat(path));
  m_eg->setFormat(std::string(f.toUtf8().data())); // It's correct?!

  emit externalGraphicChanged();

  updateUi();
}

void te::qt::widgets::LocalImagePropertyItem::showInLineContentDlg()
{
  // Bulding the filter string
  QString filter = tr("Images") + " ( ";
  QList<QByteArray> formats = QImageReader::supportedImageFormats();
  for (int i = 0; i < formats.size() - 1; ++i)
    filter += "*." + formats[i] + " ";
  filter += ")";

  QString path = QFileDialog::getOpenFileName(this, tr("Select an Image File"), "", filter);
  if (path.isNull())
    return;

  QFileInfo fInfo(path);


  te::se::InlineContent* inLine = nullptr;

  if (fInfo.completeSuffix().toStdString() == "svg")
  {
    QFile input(path);
    input.open(QIODevice::ReadOnly);

    QByteArray ba = input.readAll();

    QByteArray baBase64 = ba.toBase64();

    std::string svgXML = baBase64.data();

    inLine = new te::se::InlineContent(te::se::InlineContent::XML);
    inLine->setData(svgXML);
  }
  else
  {
    QImage img;

    // Try load image
    if (!img.load(path))
    {
      QMessageBox::critical(this, tr("Error"), tr("The selected image cannot be loaded."));
      return;
    }

    QByteArray ba;
    QBuffer bu(&ba);

    bu.open(QIODevice::WriteOnly);
    QImage scaledImg = img.scaled(m_size, m_size);

    scaledImg.save(&bu, fInfo.completeSuffix().toStdString().c_str());

    QByteArray baBase64 = ba.toBase64();

    std::string imgBase64 = baBase64.data();

    inLine = new te::se::InlineContent(te::se::InlineContent::BASE64);
    inLine->setData(imgBase64);
  }

  m_eg->setInlineContent(inLine);
  m_eg->setOnlineResource(nullptr);

  // Updating external graphic format
  QString f(QImageReader::imageFormat(path));
  m_eg->setFormat(std::string(f.toUtf8().data())); // It's correct?!

  emit externalGraphicChanged();

  updateUi();
}
